#!/usr/bin/env python

__version__ = '$Revision: 4791 $'.split()[1]
__date__ = '$Date: 2007-01-04 $'.split()[1]
__author__ = 'xmlbinmsg'

__doc__='''

Autogenerated python functions to serialize/deserialize binary messages.

Generated by: ./aisxmlbinmsg2py.py

Need to then wrap these functions with the outer AIS packet and then
convert the whole binary blob to a NMEA string.  Those functions are
not currently provided in this file.

serialize: python to ais binary
deserialize: ais binary to python

The generated code uses translators.py, binary.py, and aisstring.py
which should be packaged with the resulting files.


@requires: U{epydoc<http://epydoc.sourceforge.net/>} > 3.0alpha3
@requires: U{BitVector<http://cheeseshop.python.org/pypi/BitVector>}

@author: '''+__author__+'''
@version: ''' + __version__ +'''
@var __date__: Date of last svn commit
@undocumented: __version__ __author__ __doc__ parser
@status: under development
@license: Generated code has no license
'''

import sys
from decimal import Decimal
from BitVector import BitVector

import binary, aisstring

# FIX: check to see if these will be needed
TrueBV  = BitVector(bitstring="1")
"Why always rebuild the True bit?  This should speed things up a bunch"
FalseBV = BitVector(bitstring="0")
"Why always rebuild the False bit?  This should speed things up a bunch"


def encode(params, validate=False):
	'''Create a shipdata binary message payload to pack into an AIS Msg shipdata.

	Fields in params:
	  - MessageID(uint): AIS message number.  Must be 5 (field automatically set to "5")
	  - RepeatIndicator(uint): Indicated how many times a message has been repeated
	  - UserID(uint): Unique ship identification number (MMSI)
	  - AISversion(uint): Compliant with what edition.  0 is the first edition.
	  - IMOnumber(uint): vessel identification number (different than mmsi)
	  - callsign(aisstr6): Ship radio call sign
	  - name(aisstr6): Vessel name
	  - shipandcargo(uint): what
	  - dimA(uint): FIX: write a description
	  - dimB(uint): FIX: write a description
	  - dimC(uint): FIX: write a description
	  - dimD(uint): FIX: write a description
	  - fixtype(uint): Method used for positioning
	  - ETA(uint): FIX: break this out
	  - draught(udecimal): Maximum present static draught.  25.5 is 25.5 m or greater
	  - destination(aisstr6): Where is the vessel going
	  - dte(uint): Data terminal ready
	  - Spare(uint): Reserved for definition by a regional authority. (field automatically set to "0")
	@param params: Dictionary of field names/values.  Throws a ValueError exception if required is missing
	@param validate: Set to true to cause checking to occur.  Runs slower.  FIX: not implemented.
	@rtype: BitVector
	@return: encoded binary message (for binary messages, this needs to be wrapped in a msg 8
	@note: The returned bits may not be 6 bit aligned.  It is up to you to pad out the bits.
	'''

	bvList = []
	bvList.append(binary.setBitVectorSize(BitVector(intVal=5),6))
	if 'RepeatIndicator' in params:
		bvList.append(binary.setBitVectorSize(BitVector(intVal=params['RepeatIndicator']),2))
	else:
		bvList.append(binary.setBitVectorSize(BitVector(intVal=0),2))
	bvList.append(binary.setBitVectorSize(BitVector(intVal=params['UserID']),30))
	if 'AISversion' in params:
		bvList.append(binary.setBitVectorSize(BitVector(intVal=params['AISversion']),2))
	else:
		bvList.append(binary.setBitVectorSize(BitVector(intVal=0),2))
	if 'IMOnumber' in params:
		bvList.append(binary.setBitVectorSize(BitVector(intVal=params['IMOnumber']),30))
	else:
		bvList.append(binary.setBitVectorSize(BitVector(intVal=0),30))
	if 'callsign' in params:
		bvList.append(aisstring.encode(params['callsign'],42))
	else:
		bvList.append(aisstring.encode('@@@@@@@',42))
	if 'name' in params:
		bvList.append(aisstring.encode(params['name'],120))
	else:
		bvList.append(aisstring.encode('@@@@@@@@@@@@@@@@@@@@',120))
	if 'shipandcargo' in params:
		bvList.append(binary.setBitVectorSize(BitVector(intVal=params['shipandcargo']),8))
	else:
		bvList.append(binary.setBitVectorSize(BitVector(intVal=0),8))
	bvList.append(binary.setBitVectorSize(BitVector(intVal=params['dimA']),9))
	bvList.append(binary.setBitVectorSize(BitVector(intVal=params['dimB']),9))
	bvList.append(binary.setBitVectorSize(BitVector(intVal=params['dimC']),6))
	bvList.append(binary.setBitVectorSize(BitVector(intVal=params['dimD']),6))
	if 'fixtype' in params:
		bvList.append(binary.setBitVectorSize(BitVector(intVal=params['fixtype']),4))
	else:
		bvList.append(binary.setBitVectorSize(BitVector(intVal=0),4))
	bvList.append(binary.setBitVectorSize(BitVector(intVal=params['ETA']),20))
	if 'draught' in params:
		bvList.append(binary.setBitVectorSize(BitVector(intVal=int((Decimal(params['draught'])*Decimal('10')))),8))
	else:
		bvList.append(binary.setBitVectorSize(BitVector(intVal=int(0)),8))
	if 'destination' in params:
		bvList.append(aisstring.encode(params['destination'],120))
	else:
		bvList.append(aisstring.encode('@@@@@@@@@@@@@@@@@@@@',120))
	bvList.append(binary.setBitVectorSize(BitVector(intVal=params['dte']),1))
	bvList.append(binary.setBitVectorSize(BitVector(intVal=0),1))

	return binary.joinBV(bvList)

def decode(bv, validate=False):
	'''Unpack a shipdata message 

	Fields in params:
	  - MessageID(uint): AIS message number.  Must be 5 (field automatically set to "5")
	  - RepeatIndicator(uint): Indicated how many times a message has been repeated
	  - UserID(uint): Unique ship identification number (MMSI)
	  - AISversion(uint): Compliant with what edition.  0 is the first edition.
	  - IMOnumber(uint): vessel identification number (different than mmsi)
	  - callsign(aisstr6): Ship radio call sign
	  - name(aisstr6): Vessel name
	  - shipandcargo(uint): what
	  - dimA(uint): FIX: write a description
	  - dimB(uint): FIX: write a description
	  - dimC(uint): FIX: write a description
	  - dimD(uint): FIX: write a description
	  - fixtype(uint): Method used for positioning
	  - ETA(uint): FIX: break this out
	  - draught(udecimal): Maximum present static draught.  25.5 is 25.5 m or greater
	  - destination(aisstr6): Where is the vessel going
	  - dte(uint): Data terminal ready
	  - Spare(uint): Reserved for definition by a regional authority. (field automatically set to "0")
	@type bv: BitVector
	@param bv: Bits defining a message
	@param validate: Set to true to cause checking to occur.  Runs slower.  FIX: not implemented.
	@rtype: dict
	@return: params
	'''

	#Would be nice to check the bit count here..
	#if validate:
	#	assert (len(bv)==FIX: SOME NUMBER)
	r = {}
	r['MessageID']=5
	r['RepeatIndicator']=int(bv[6:8])
	r['UserID']=int(bv[8:38])
	r['AISversion']=int(bv[38:40])
	r['IMOnumber']=int(bv[40:70])
	r['callsign']=aisstring.decode(bv[70:112])
	r['name']=aisstring.decode(bv[112:232])
	r['shipandcargo']=int(bv[232:240])
	r['dimA']=int(bv[240:249])
	r['dimB']=int(bv[249:258])
	r['dimC']=int(bv[258:264])
	r['dimD']=int(bv[264:270])
	r['fixtype']=int(bv[270:274])
	r['ETA']=int(bv[274:294])
	r['draught']=Decimal(int(bv[294:302]))/Decimal('10')
	r['destination']=aisstring.decode(bv[302:422])
	r['dte']=int(bv[422:423])
	r['Spare']=0
	return r

def decodeMessageID(bv, validate=False):
	return 5

def decodeRepeatIndicator(bv, validate=False):
	return int(bv[6:8])

def decodeUserID(bv, validate=False):
	return int(bv[8:38])

def decodeAISversion(bv, validate=False):
	return int(bv[38:40])

def decodeIMOnumber(bv, validate=False):
	return int(bv[40:70])

def decodecallsign(bv, validate=False):
	return aisstring.decode(bv[70:112])

def decodename(bv, validate=False):
	return aisstring.decode(bv[112:232])

def decodeshipandcargo(bv, validate=False):
	return int(bv[232:240])

def decodedimA(bv, validate=False):
	return int(bv[240:249])

def decodedimB(bv, validate=False):
	return int(bv[249:258])

def decodedimC(bv, validate=False):
	return int(bv[258:264])

def decodedimD(bv, validate=False):
	return int(bv[264:270])

def decodefixtype(bv, validate=False):
	return int(bv[270:274])

def decodeETA(bv, validate=False):
	return int(bv[274:294])

def decodedraught(bv, validate=False):
	return Decimal(int(bv[294:302]))/Decimal('10')

def decodedestination(bv, validate=False):
	return aisstring.decode(bv[302:422])

def decodedte(bv, validate=False):
	return int(bv[422:423])

def decodeSpare(bv, validate=False):
	return 0

def printFields(params, out=sys.stdout, format='std'):
	'''Print a shipdata message to stdout.

	Fields in params:
	  - MessageID(uint): AIS message number.  Must be 5 (field automatically set to "5")
	  - RepeatIndicator(uint): Indicated how many times a message has been repeated
	  - UserID(uint): Unique ship identification number (MMSI)
	  - AISversion(uint): Compliant with what edition.  0 is the first edition.
	  - IMOnumber(uint): vessel identification number (different than mmsi)
	  - callsign(aisstr6): Ship radio call sign
	  - name(aisstr6): Vessel name
	  - shipandcargo(uint): what
	  - dimA(uint): FIX: write a description
	  - dimB(uint): FIX: write a description
	  - dimC(uint): FIX: write a description
	  - dimD(uint): FIX: write a description
	  - fixtype(uint): Method used for positioning
	  - ETA(uint): FIX: break this out
	  - draught(udecimal): Maximum present static draught.  25.5 is 25.5 m or greater
	  - destination(aisstr6): Where is the vessel going
	  - dte(uint): Data terminal ready
	  - Spare(uint): Reserved for definition by a regional authority. (field automatically set to "0")
	@param params: Dictionary of field names/values.  
	@param out: File like object to write to
	@rtype: stdout
	@return: text to out
	'''

	if 'std'==format:
		out.write("shipdata:\n")
		if 'MessageID' in params: out.write("	MessageID:        "+str(params['MessageID'])+"\n")
		if 'RepeatIndicator' in params: out.write("	RepeatIndicator:  "+str(params['RepeatIndicator'])+"\n")
		if 'UserID' in params: out.write("	UserID:           "+str(params['UserID'])+"\n")
		if 'AISversion' in params: out.write("	AISversion:       "+str(params['AISversion'])+"\n")
		if 'IMOnumber' in params: out.write("	IMOnumber:        "+str(params['IMOnumber'])+"\n")
		if 'callsign' in params: out.write("	callsign:         "+str(params['callsign'])+"\n")
		if 'name' in params: out.write("	name:             "+str(params['name'])+"\n")
		if 'shipandcargo' in params: out.write("	shipandcargo:     "+str(params['shipandcargo'])+"\n")
		if 'dimA' in params: out.write("	dimA:             "+str(params['dimA'])+"\n")
		if 'dimB' in params: out.write("	dimB:             "+str(params['dimB'])+"\n")
		if 'dimC' in params: out.write("	dimC:             "+str(params['dimC'])+"\n")
		if 'dimD' in params: out.write("	dimD:             "+str(params['dimD'])+"\n")
		if 'fixtype' in params: out.write("	fixtype:          "+str(params['fixtype'])+"\n")
		if 'ETA' in params: out.write("	ETA:              "+str(params['ETA'])+"\n")
		if 'draught' in params: out.write("	draught:          "+str(params['draught'])+"\n")
		if 'destination' in params: out.write("	destination:      "+str(params['destination'])+"\n")
		if 'dte' in params: out.write("	dte:              "+str(params['dte'])+"\n")
		if 'Spare' in params: out.write("	Spare:            "+str(params['Spare'])+"\n")
	elif 'html'==format:
		out.write("<h3>shipdata<h3>\n")
		out.write("<table border=\"1\">\n")
		out.write("<tr bgcolor=\"orange\">\n")
		out.write("<th align=\"left\">Field Name</th>\n")
		out.write("<th align=\"left\">Type</th>\n")
		out.write("<th align=\"left\">Value</th>\n")
		out.write("<th align=\"left\">Value in Lookup Table</th>\n")
		out.write("\n")
		out.write("<tr>\n")
		out.write("<td>MessageID</td>\n")
		out.write("<td>uint</td>\n")
		if 'MessageID' in params:
			out.write("	<td>"+str(params['MessageID'])+"</td>\n")
			out.write("	<td>"+str(params['MessageID'])+"</td>\n")
		out.write("</tr>\n")
		out.write("\n")
		out.write("<tr>\n")
		out.write("<td>RepeatIndicator</td>\n")
		out.write("<td>uint</td>\n")
		if 'RepeatIndicator' in params:
			out.write("	<td>"+str(params['RepeatIndicator'])+"</td>\n")
			if str(params['RepeatIndicator']) in RepeatIndicatorDecodeLut:
				out.write("<td>"+RepeatIndicatorDecodeLut[str(params['RepeatIndicator'])]+"</td>")
			else:
				out.write("<td><i>Missing LUT entry</i></td>")
		out.write("</tr>\n")
		out.write("\n")
		out.write("<tr>\n")
		out.write("<td>UserID</td>\n")
		out.write("<td>uint</td>\n")
		if 'UserID' in params:
			out.write("	<td>"+str(params['UserID'])+"</td>\n")
			out.write("	<td>"+str(params['UserID'])+"</td>\n")
		out.write("</tr>\n")
		out.write("\n")
		out.write("<tr>\n")
		out.write("<td>AISversion</td>\n")
		out.write("<td>uint</td>\n")
		if 'AISversion' in params:
			out.write("	<td>"+str(params['AISversion'])+"</td>\n")
			out.write("	<td>"+str(params['AISversion'])+"</td>\n")
		out.write("</tr>\n")
		out.write("\n")
		out.write("<tr>\n")
		out.write("<td>IMOnumber</td>\n")
		out.write("<td>uint</td>\n")
		if 'IMOnumber' in params:
			out.write("	<td>"+str(params['IMOnumber'])+"</td>\n")
			out.write("	<td>"+str(params['IMOnumber'])+"</td>\n")
		out.write("</tr>\n")
		out.write("\n")
		out.write("<tr>\n")
		out.write("<td>callsign</td>\n")
		out.write("<td>aisstr6</td>\n")
		if 'callsign' in params:
			out.write("	<td>"+str(params['callsign'])+"</td>\n")
			out.write("	<td>"+str(params['callsign'])+"</td>\n")
		out.write("</tr>\n")
		out.write("\n")
		out.write("<tr>\n")
		out.write("<td>name</td>\n")
		out.write("<td>aisstr6</td>\n")
		if 'name' in params:
			out.write("	<td>"+str(params['name'])+"</td>\n")
			out.write("	<td>"+str(params['name'])+"</td>\n")
		out.write("</tr>\n")
		out.write("\n")
		out.write("<tr>\n")
		out.write("<td>shipandcargo</td>\n")
		out.write("<td>uint</td>\n")
		if 'shipandcargo' in params:
			out.write("	<td>"+str(params['shipandcargo'])+"</td>\n")
			out.write("	<td>"+str(params['shipandcargo'])+"</td>\n")
		out.write("</tr>\n")
		out.write("\n")
		out.write("<tr>\n")
		out.write("<td>dimA</td>\n")
		out.write("<td>uint</td>\n")
		if 'dimA' in params:
			out.write("	<td>"+str(params['dimA'])+"</td>\n")
			out.write("	<td>"+str(params['dimA'])+"</td>\n")
		out.write("</tr>\n")
		out.write("\n")
		out.write("<tr>\n")
		out.write("<td>dimB</td>\n")
		out.write("<td>uint</td>\n")
		if 'dimB' in params:
			out.write("	<td>"+str(params['dimB'])+"</td>\n")
			out.write("	<td>"+str(params['dimB'])+"</td>\n")
		out.write("</tr>\n")
		out.write("\n")
		out.write("<tr>\n")
		out.write("<td>dimC</td>\n")
		out.write("<td>uint</td>\n")
		if 'dimC' in params:
			out.write("	<td>"+str(params['dimC'])+"</td>\n")
			out.write("	<td>"+str(params['dimC'])+"</td>\n")
		out.write("</tr>\n")
		out.write("\n")
		out.write("<tr>\n")
		out.write("<td>dimD</td>\n")
		out.write("<td>uint</td>\n")
		if 'dimD' in params:
			out.write("	<td>"+str(params['dimD'])+"</td>\n")
			out.write("	<td>"+str(params['dimD'])+"</td>\n")
		out.write("</tr>\n")
		out.write("\n")
		out.write("<tr>\n")
		out.write("<td>fixtype</td>\n")
		out.write("<td>uint</td>\n")
		if 'fixtype' in params:
			out.write("	<td>"+str(params['fixtype'])+"</td>\n")
			if str(params['fixtype']) in fixtypeDecodeLut:
				out.write("<td>"+fixtypeDecodeLut[str(params['fixtype'])]+"</td>")
			else:
				out.write("<td><i>Missing LUT entry</i></td>")
		out.write("</tr>\n")
		out.write("\n")
		out.write("<tr>\n")
		out.write("<td>ETA</td>\n")
		out.write("<td>uint</td>\n")
		if 'ETA' in params:
			out.write("	<td>"+str(params['ETA'])+"</td>\n")
			out.write("	<td>"+str(params['ETA'])+"</td>\n")
		out.write("</tr>\n")
		out.write("\n")
		out.write("<tr>\n")
		out.write("<td>draught</td>\n")
		out.write("<td>udecimal</td>\n")
		if 'draught' in params:
			out.write("	<td>"+str(params['draught'])+"</td>\n")
			out.write("	<td>"+str(params['draught'])+"</td>\n")
		out.write("</tr>\n")
		out.write("\n")
		out.write("<tr>\n")
		out.write("<td>destination</td>\n")
		out.write("<td>aisstr6</td>\n")
		if 'destination' in params:
			out.write("	<td>"+str(params['destination'])+"</td>\n")
			out.write("	<td>"+str(params['destination'])+"</td>\n")
		out.write("</tr>\n")
		out.write("\n")
		out.write("<tr>\n")
		out.write("<td>dte</td>\n")
		out.write("<td>uint</td>\n")
		if 'dte' in params:
			out.write("	<td>"+str(params['dte'])+"</td>\n")
			if str(params['dte']) in dteDecodeLut:
				out.write("<td>"+dteDecodeLut[str(params['dte'])]+"</td>")
			else:
				out.write("<td><i>Missing LUT entry</i></td>")
		out.write("</tr>\n")
		out.write("\n")
		out.write("<tr>\n")
		out.write("<td>Spare</td>\n")
		out.write("<td>uint</td>\n")
		if 'Spare' in params:
			out.write("	<td>"+str(params['Spare'])+"</td>\n")
			out.write("	<td>"+str(params['Spare'])+"</td>\n")
		out.write("</tr>\n")
		out.write("</table>\n")
	else: 
		print "ERROR: unknown format:",format
		assert False

	return # Nothing to return

RepeatIndicatorEncodeLut = {
	'default':'0',
	'do not repeat any more':'3',
	} #RepeatIndicatorEncodeLut

RepeatIndicatorDecodeLut = {
	'0':'default',
	'3':'do not repeat any more',
	} # RepeatIndicatorEncodeLut

fixtypeEncodeLut = {
	'undefined':'0',
	'GPS':'1',
	'GLONASS':'2',
	'combined GPS/GLONASS':'3',
	'Loran-C':'4',
	'Chayka':'5',
	'integrated navigation system':'6',
	'surveyed':'7',
	} #fixtypeEncodeLut

fixtypeDecodeLut = {
	'0':'undefined',
	'1':'GPS',
	'2':'GLONASS',
	'3':'combined GPS/GLONASS',
	'4':'Loran-C',
	'5':'Chayka',
	'6':'integrated navigation system',
	'7':'surveyed',
	} # fixtypeEncodeLut

dteEncodeLut = {
	'available':'0',
	'not available':'1',
	} #dteEncodeLut

dteDecodeLut = {
	'0':'available',
	'1':'not available',
	} # dteEncodeLut



######################################################################
# UNIT TESTING
######################################################################
import unittest
def testParams():
	'''Return a params file base on the testvalue tags.
	@rtype: dict
	@return: params based on testvalue tags
	'''
	params = {}
	params['MessageID'] = 5
	params['RepeatIndicator'] = 1
	params['UserID'] = 1193046
	params['AISversion'] = 0
	params['IMOnumber'] = 3210
	params['callsign'] = 'PIRATE1'
	params['name'] = 'BLACK PEARL@@@@@@@@@'
	params['shipandcargo'] = 55
	params['dimA'] = 10
	params['dimB'] = 11
	params['dimC'] = 12
	params['dimD'] = 13
	params['fixtype'] = 1
	params['ETA'] = 0
	params['draught'] = Decimal('21.1')
	params['destination'] = 'NOWHERE@@@@@@@@@@@@@'
	params['dte'] = 0
	params['Spare'] = 0

	return params

class Testshipdata(unittest.TestCase):
	'''Use testvalue tag text from each type to build test case the shipdata message'''
	def testEncodeDecode(self):

		params = testParams()
		bits   = encode(params)
		r      = decode(bits)

		# Check that each parameter came through ok.
		self.failUnlessEqual(r['MessageID'],params['MessageID'])
		self.failUnlessEqual(r['RepeatIndicator'],params['RepeatIndicator'])
		self.failUnlessEqual(r['UserID'],params['UserID'])
		self.failUnlessEqual(r['AISversion'],params['AISversion'])
		self.failUnlessEqual(r['IMOnumber'],params['IMOnumber'])
		self.failUnlessEqual(r['callsign'],params['callsign'])
		self.failUnlessEqual(r['name'],params['name'])
		self.failUnlessEqual(r['shipandcargo'],params['shipandcargo'])
		self.failUnlessEqual(r['dimA'],params['dimA'])
		self.failUnlessEqual(r['dimB'],params['dimB'])
		self.failUnlessEqual(r['dimC'],params['dimC'])
		self.failUnlessEqual(r['dimD'],params['dimD'])
		self.failUnlessEqual(r['fixtype'],params['fixtype'])
		self.failUnlessEqual(r['ETA'],params['ETA'])
		self.failUnlessAlmostEqual(r['draught'],params['draught'],3)
		self.failUnlessEqual(r['destination'],params['destination'])
		self.failUnlessEqual(r['dte'],params['dte'])
		self.failUnlessEqual(r['Spare'],params['Spare'])

def addMsgOptions(parser):
	parser.add_option('-d','--decode',dest='doDecode',default=False,action='store_true',
		help='decode a "shipdata" AIS message')
	parser.add_option('-e','--encode',dest='doEncode',default=False,action='store_true',
		help='encode a "shipdata" AIS message')
	parser.add_option('--RepeatIndicator-field', dest='RepeatIndicatorField',default=0,metavar='uint',type='int'
		,help='Field parameter value [default: %default]')
	parser.add_option('--UserID-field', dest='UserIDField',metavar='uint',type='int'
		,help='Field parameter value [default: %default]')
	parser.add_option('--AISversion-field', dest='AISversionField',default=0,metavar='uint',type='int'
		,help='Field parameter value [default: %default]')
	parser.add_option('--IMOnumber-field', dest='IMOnumberField',default=0,metavar='uint',type='int'
		,help='Field parameter value [default: %default]')
	parser.add_option('--callsign-field', dest='callsignField',default='@@@@@@@',metavar='aisstr6',type='string'
		,help='Field parameter value [default: %default]')
	parser.add_option('--name-field', dest='nameField',default='@@@@@@@@@@@@@@@@@@@@',metavar='aisstr6',type='string'
		,help='Field parameter value [default: %default]')
	parser.add_option('--shipandcargo-field', dest='shipandcargoField',default=0,metavar='uint',type='int'
		,help='Field parameter value [default: %default]')
	parser.add_option('--dimA-field', dest='dimAField',metavar='uint',type='int'
		,help='Field parameter value [default: %default]')
	parser.add_option('--dimB-field', dest='dimBField',metavar='uint',type='int'
		,help='Field parameter value [default: %default]')
	parser.add_option('--dimC-field', dest='dimCField',metavar='uint',type='int'
		,help='Field parameter value [default: %default]')
	parser.add_option('--dimD-field', dest='dimDField',metavar='uint',type='int'
		,help='Field parameter value [default: %default]')
	parser.add_option('--fixtype-field', dest='fixtypeField',default=0,metavar='uint',type='int'
		,help='Field parameter value [default: %default]')
	parser.add_option('--ETA-field', dest='ETAField',metavar='uint',type='int'
		,help='Field parameter value [default: %default]')
	parser.add_option('--draught-field', dest='draughtField',default=Decimal('0'),metavar='udecimal',type='string'
		,help='Field parameter value [default: %default]')
	parser.add_option('--destination-field', dest='destinationField',default='@@@@@@@@@@@@@@@@@@@@',metavar='aisstr6',type='string'
		,help='Field parameter value [default: %default]')
	parser.add_option('--dte-field', dest='dteField',metavar='uint',type='int'
		,help='Field parameter value [default: %default]')

############################################################
if __name__=='__main__':

	from optparse import OptionParser
	parser = OptionParser(usage="%prog [options]",
		version="%prog "+__version__)

	parser.add_option('--doc-test',dest='doctest',default=False,action='store_true',
		help='run the documentation tests')
	parser.add_option('--unit-test',dest='unittest',default=False,action='store_true',
		help='run the unit tests')
	parser.add_option('-v','--verbose',dest='verbose',default=False,action='store_true',
		help='Make the test output verbose')

	# FIX: remove nmea from binary messages.  No way to build the whole packet?
	# FIX: or build the surrounding msg 8 for a broadcast?
	typeChoices = ('binary','nmeapayload','nmea') # FIX: what about a USCG type message?
	parser.add_option('-t','--type',choices=typeChoices,type='choice',dest='ioType'
		,default='nmeapayload'
		,help='What kind of string to expect ('+', '.join(typeChoices)+') [default: %default]')

	outputChoices = ('std','html','xml')
	parser.add_option('-T','--output-type',choices=outputChoices,type='choice',dest='outputType'
		,default='std'
		,help='What kind of string to output ('+', '.join(outputChoices)+') [default: %default]')

	parser.add_option('-o','--output',dest='outputFileName',default=None,
		      help='Name of the python file to write [default: stdout]')

	addMsgOptions(parser)

	(options,args) = parser.parse_args()
	success=True

	if options.doctest:
		import os; print os.path.basename(sys.argv[0]), 'doctests ...',
		sys.argv= [sys.argv[0]]
		if options.verbose: sys.argv.append('-v')
		import doctest
		numfail,numtests=doctest.testmod()
		if numfail==0: print 'ok'
		else: 
			print 'FAILED'
			success=False

	if not success: sys.exit('Something Failed')
	del success # Hide success from epydoc

	if options.unittest:
		sys.argv = [sys.argv[0]]
		if options.verbose: sys.argv.append('-v')
		unittest.main()

	outfile = sys.stdout
	if None!=options.outputFileName:
		outfile = file(options.outputFileName,'w')


	if options.doEncode:
		# First make sure all non required options are specified
		if None==options.RepeatIndicatorField: parser.error("missing value for RepeatIndicatorField")
		if None==options.UserIDField: parser.error("missing value for UserIDField")
		if None==options.AISversionField: parser.error("missing value for AISversionField")
		if None==options.IMOnumberField: parser.error("missing value for IMOnumberField")
		if None==options.callsignField: parser.error("missing value for callsignField")
		if None==options.nameField: parser.error("missing value for nameField")
		if None==options.shipandcargoField: parser.error("missing value for shipandcargoField")
		if None==options.dimAField: parser.error("missing value for dimAField")
		if None==options.dimBField: parser.error("missing value for dimBField")
		if None==options.dimCField: parser.error("missing value for dimCField")
		if None==options.dimDField: parser.error("missing value for dimDField")
		if None==options.fixtypeField: parser.error("missing value for fixtypeField")
		if None==options.ETAField: parser.error("missing value for ETAField")
		if None==options.draughtField: parser.error("missing value for draughtField")
		if None==options.destinationField: parser.error("missing value for destinationField")
		if None==options.dteField: parser.error("missing value for dteField")
		msgDict={
			'MessageID': '5',
			'RepeatIndicator': options.RepeatIndicatorField,
			'UserID': options.UserIDField,
			'AISversion': options.AISversionField,
			'IMOnumber': options.IMOnumberField,
			'callsign': options.callsignField,
			'name': options.nameField,
			'shipandcargo': options.shipandcargoField,
			'dimA': options.dimAField,
			'dimB': options.dimBField,
			'dimC': options.dimCField,
			'dimD': options.dimDField,
			'fixtype': options.fixtypeField,
			'ETA': options.ETAField,
			'draught': options.draughtField,
			'destination': options.destinationField,
			'dte': options.dteField,
			'Spare': '0',
		}

		bits = encode(msgDict)
		if 'binary'==options.ioType: print str(bits)
		elif 'nmeapayload'==options.ioType:
		        # FIX: figure out if this might be necessary at compile time
			print "bitLen",len(bits)
			bitLen=len(bits)
			if bitLen%6!=0:
			    bits = bits + BitVector(size=(6 - (bitLen%6)))  # Pad out to multiple of 6
			print "result:",binary.bitvectoais6(bits)[0]


		# FIX: Do not emit this option for the binary message payloads.  Does not make sense.
		elif 'nmea'==options.ioType: sys.exit("FIX: need to implement this capability")
		else: sys.exit('ERROR: unknown ioType.  Help!')

	if options.doDecode:
		for msg in args:
			bv = None
			if    'binary'    == options.ioType: bv = BitVector(bitstring=msg)
			elif 'nmeapayload'== options.ioType: bv = binary.ais6tobitvec(msg)
			elif 'nmea'       == options.ioType: bv = binary.ais6tobitvec(msg.split(',')[5])
			else: sys.exit('ERROR: unknown ioType.  Help!')

			printFields(decode(bv),out=outfile,format=options.outputType)
